/********************************************************
 *  ██████╗  ██████╗████████╗██╗
 * ██╔════╝ ██╔════╝╚══██╔══╝██║
 * ██║  ███╗██║        ██║   ██║
 * ██║   ██║██║        ██║   ██║
 * ╚██████╔╝╚██████╗   ██║   ███████╗
 *  ╚═════╝  ╚═════╝   ╚═╝   ╚══════╝
 * Geophysical Computational Tools & Library (GCTL)
 *
 * Copyright (c) 2023  Yi Zhang (yizhang-geo@zju.edu.cn)
 *
 * GCTL is distributed under a dual licensing scheme. You can redistribute 
 * it and/or modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation, either version 2 
 * of the License, or (at your option) any later version. You should have 
 * received a copy of the GNU Lesser General Public License along with this 
 * program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * If the terms and conditions of the LGPL v.2. would prevent you from using 
 * the GCTL, please consider the option to obtain a commercial license for a 
 * fee. These licenses are offered by the GCTL's original author. As a rule, 
 * licenses are provided "as-is", unlimited in time for a one time fee. Please 
 * send corresponding requests to: yizhang-geo@zju.edu.cn. Please do not forget 
 * to include some description of your company and the realm of its activities. 
 * Also add information on how to contact you by electronic and paper mail.
 ******************************************************/

#include "../lib/core.h"
#include "../lib/io.h"

using namespace gctl;

int main(int argc, char const *argv[]) try
{
    mesh_io mshio;

/*  
    mshio.read_tetgen_ascii("tmp/ex1.1");
    mshio.edit_group(Disable, GeometryTag, 5);
    mshio.edit_group(GeometryTag, 1, PhysicalTag, 1);
    mshio.edit_group(GeometryTag, 2, PhysicalTag, 2);
    mshio.edit_group(GeometryTag, 3, PhysicalTag, 3);
    mshio.edit_group(GeometryTag, 4, PhysicalTag, 4);
    mshio.edit_group(GeometryTag, 1, "Boundary");
    mshio.edit_group(GeometryTag, 2, "Body1");
    mshio.edit_group(GeometryTag, 3, "Body2");
    mshio.edit_group(GeometryTag, 4, "Body3");
    mshio.save_gmsh_v2_ascii("tmp/ex1.1");
*/

    mshio.read_gmsh_v2_ascii("tmp/ex1.1");
    mshio.convert_tags_to_data(GeometryTag);

    array<double> body_val(mshio.element_size("Body2"), 2.0);
    mshio.add_element_data(body_val, "BodyValue", "Body2");

    array<double> body_val2(mshio.element_size("Body3"), 1.0);
    mshio.add_element_data(body_val2, "BodyValue", "Body3");

    mshio.save_gmsh_v2_ascii("tmp/ex1.2");
    //mshio.save_vtk_legacy_ascii("tmp/ex1.1");
    mshio.info();

    const array<vertex3dc> &nodes = mshio.get_nodes();

    array<tetrahedron> body2_tets;
    mshio.export_elements_to(body2_tets, "All");

    gmshio gio;
    gio.init_file("tmp.msh", Output);
    gio.set_packed(NotPacked, Output);
    gio.save_mesh(body2_tets, nodes);
/*
    mshio.read_gmsh_v2_ascii("tmp/wjb.1");
    mshio.edit_group(Disable);
    mshio.edit_group(Enable, GeometryTag, 3);
    mshio.edit_group(Enable, GeometryTag, 8);
    mshio.edit_group(Enable, GeometryTag, 9);

    mshio.save_gmsh_v2_ascii("tmp/wjb.2");
*/
    return 0;
}
catch(std::exception &e)
{
    GCTL_ShowWhatError(e.what(), GCTL_ERROR_ERROR, 0, 0, 0);
}